/*   Copyright (c) AT&T Corp.  All rights reserved.
   
This software may only be used by you under license from 
AT&T Corp. ("AT&T").  A copy of AT&T's Source Code Agreement 
is available at AT&T's Internet website having the URL 

http://www.research.att.com/sw/tools/graphviz/license/

If you received this software without first entering into a license 
with AT&T, you have an infringing copy of this software and cannot 
use it without violating AT&T's intellectual property rights. */

struct View;
typedef map<DString,View*> Views;
extern Views g_views; // in incrcmds
struct View {
	typedef map<DString,Layout::Node*> nodeDict;
	typedef map<DString,Layout::Edge*> edgeDict;
	nodeDict nodes;
	edgeDict edges;
	Layout layout,current;
	ChangeQueue Q;
	Server *dgserver;
	int locks;
	bool allowAnOpen; // when created outside incrface but want to allow "open view"

	// implement this to respond to incrface events
	virtual void IncrHappened() = 0;
	virtual void IncrNewNode(Layout::Node *n) = 0;
	virtual void IncrNewEdge(Layout::Edge *e) = 0;

	View(Name name = Name()) : 
	  current(&layout),
	  Q(&layout,&current),
	  dgserver(0),
	  locks(0),
	  allowAnOpen(false) {
		if(name.empty())
			name = randomName('v');
		gd<Name>(&layout) = name;
		g_views[name] = this;
	}
	virtual ~View() {
		if(dgserver)
			delete dgserver;
	}
	// tricky problem: if client doesn't specify a name, we need to generate one
	// can't use e.g. e17 because the client might already have something named that!
	// so generate a random name
	static DString randomName(char prefix) {
		char buf[10],i=2;
		unsigned char ch;
		buf[0] = prefix;
		buf[1] = '_';
		while(i<8) 
			if(isalnum(ch = rand()%256))
				buf[i++] = ch;
		buf[8] = 0;
		return buf;
	}
	// to detect those bad names that agraph is generating
	static bool poorEdgeName(const char *name) {
		if(!name[0])
			return true;
		if(name[0]!='e')
			return false;
		if(!name[1])
			return false; // "e" okay i guess
		for(int i=1;name[i]; ++i)
			if(!isdigit(name[i]))
				return false;
		return true;
	}
	pair<Layout::Node*,bool> getNode(DString id,bool create) {
		if(id.empty())
			id = randomName('n');
		Layout::Node *n = nodes[id];
		if(n)
			return make_pair(n,false);
		if(!create) 
			return make_pair<Layout::Node*>(0,false);
		n = layout.create_node();
		gd<Name>(n) = id;
		nodes[id] = n;
		return make_pair(n,true);
	}
	Layout::Node *getNode(DString id) {
		return getNode(id,false).first;
	}
	pair<Layout::Edge*,bool> getEdge(DString id,Layout::Node *t,Layout::Node *h,bool create) {
		Layout::Edge *e=0;
		if(id.empty())
			id = randomName('e');
		else if((e = edges[id]))
			return make_pair(e,false);
		if(!(t && h))
			return make_pair<Layout::Edge*>(0,false);
		e = layout.find_edge(t,h);
		if(e)
			return make_pair(e,false);
		if(!create)
			return make_pair<Layout::Edge*>(0,false);
		e = layout.create_edge(t,h).first;
		gd<Name>(e) = id;
		edges[id] = e;
		return make_pair(e,true);
	}
	pair<Layout::Edge*,bool> getEdge(DString id,DString tail,DString head,bool create) {
		assert(tail.size()&&head.size());
		Layout::Node *t = getNode(tail,false).first,
			*h = getNode(head,false).first;
		return getEdge(id,t,h,create);
	}
	Layout::Edge *getEdge(DString id,DString tail,DString head) {
		return getEdge(id,tail,head,false).first;
	}
	Layout::Edge *getEdge(DString id) {
		return getEdge(id,(Layout::Node*)0,(Layout::Node*)0,false).first;
	}
	void rename(Layout *l,DString newName) {
		assert(l==&layout);
		g_views.erase(gd<Name>(&layout));
		g_views[gd<Name>(&layout) = newName] = this;
	}
	void rename(Layout::Node *n,DString newName) {
		forget(n);
		nodes[gd<Name>(n) = newName] = n;
	}
	void rename(Layout::Edge *e,DString newName) {
		forget(e);
		edges[gd<Name>(e) = newName] = e;
	}
	void forget(Layout::Node *n) {
		nodes.erase(gd<Name>(n));
	}
	void forget(Layout::Edge *e) {
		edges.erase(gd<Name>(e));
	}
};
